﻿using Microsoft.Win32;
using SLGraphClient;
using SLStreamerPro;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace SLStreamerTest
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public static readonly DependencyProperty SchedulerAddressProperty = DependencyProperty.Register("SchedulerAddress", typeof(string), typeof(MainWindow), new PropertyMetadata(string.Empty));
        public string SchedulerAddress
        {
            get { return (string)GetValue(SchedulerAddressProperty); }
            set { SetValue(SchedulerAddressProperty, value); }
        }

        private static string WindowTitle = "SLStreamerTest";
        private static string ConnectStr = "Connect";
        private static string DisconnectStr = "Disconnect";

        SLGScheduler _scheduler = new SLGScheduler();

        public MainWindow()
        {
            InitializeComponent();
            this.Title = WindowTitle;
            try
            {
                using (RegistryKey key = Registry.CurrentUser.OpenSubKey("Software\\SoftLab-NSK\\SLStreamerTest"))
                {
                    if (key != null)
                    {
                        this.SchedulerAddress = key.GetValue("SchedulerAddress", string.Empty) as string;
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, this.Title, MessageBoxButton.OK, MessageBoxImage.Error);
            }
        
        }

        public void Connect()
        {
          
            try
            {
                try
                {
                    if (_scheduler.ConnectWithException(SchedulerAddress) != HR.S_OK)
                    {
                        return;
                    }
                }
                catch (EndpointNotFoundException)
                {

                    MessageBox.Show("Unable to connect", WindowTitle, MessageBoxButton.OK, MessageBoxImage.Error);
                    return;
                }
                catch (System.Exception ex)
                {
                    MessageBox.Show(ex.Message, WindowTitle, MessageBoxButton.OK, MessageBoxImage.Error);
                    return;
                }

                btnConnect.Content = DisconnectStr;
				
				UpdateServers();
                //UpdateTasks();
                //UpdateSchedule();
			}
			catch (Exception ex)
			{
                MessageBox.Show(ex.Message, WindowTitle, MessageBoxButton.OK, MessageBoxImage.Error);
                Disconnect();
			}
			finally
			{
                if (_scheduler.IsConnected)
                {
                    this.Title = string.Format("Connected to {0}", this.SchedulerAddress);
                }
                else
                {
                    this.Title = WindowTitle;
                }
			}
        }

        private void Disconnect()
        {
            try
            {
                if (_scheduler.IsConnected)
                {
                    _scheduler.Disconnect();
                    _serversTreeView.Items.Clear();
                    btnConnect.Content = ConnectStr;
                    this.Title = WindowTitle;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, WindowTitle, MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void UpdateServers()
        {
            _serversTreeView.Items.Clear();
            IEnumerator serverEnum;
            _scheduler.GetServersEnumerator(out serverEnum);
            serverEnum.Reset();
            while (serverEnum.MoveNext())
            {
                SLGServer server = serverEnum.Current as SLGServer;
                TreeViewItem serverItem = new TreeViewItem();
                serverItem.Tag = server;
                serverItem.Header = server.Name;
                serverItem.IsExpanded = true;
                _serversTreeView.Items.Add(serverItem);
                UpdateGraphs(serverItem);
            }
        }

        private void UpdateGraphs(TreeViewItem serverItem)
        {
            if (serverItem == null)
            {
                return;
            }
            SLGServer server = serverItem.Tag as SLGServer;
            if (server == null)
            {
                return;
            }
            server.UpdateGraphs();

            IEnumerator<SLGGraph> graphEnum = server.GetGraphsEnumerator();
            graphEnum.Reset();
            while (graphEnum.MoveNext())
            {
                TreeViewItem currGraphItem = null;
                foreach (TreeViewItem serverGraphItem in serverItem.Items)
                {
                    SLGGraph graph = serverGraphItem.Tag as SLGGraph;
                    if (graph == null)
                        continue;
                    if (string.Compare(graph.Name, graphEnum.Current.Name) == 0)
                    {
                        currGraphItem = serverGraphItem;
                        graphEnum.Current.Update();
                        currGraphItem.Tag = graphEnum.Current;
                        currGraphItem.Header = graphEnum.Current.Name;
                        break;
                    }
                }
                if (currGraphItem == null)
                {
                    graphEnum.Current.Update();
                    if (graphEnum.Current.State == SLGGraphState.Running)
                    {
                        currGraphItem = new TreeViewItem();
                        currGraphItem.Tag = graphEnum.Current;
                        currGraphItem.Header = graphEnum.Current.Name;
                        MenuItem mi = new MenuItem();
                        mi.Header = "View ouput device statistic";
                        mi.Click += new RoutedEventHandler(_viewStatButton_Click);
                        currGraphItem.ContextMenu = new System.Windows.Controls.ContextMenu();
                        currGraphItem.ContextMenu.Items.Add(mi);
                        serverItem.Items.Add(currGraphItem);
                    }
                }
            }
        }

        private void _viewStatButton_Click(object sender, RoutedEventArgs e)
        {
            if (_serversTreeView.SelectedItem == null)
            {
                return;
            }
            TreeViewItem graphItem = _serversTreeView.SelectedItem as TreeViewItem;
            if (graphItem == null ||
                graphItem.Tag == null)
            {
                return;
            }
            SLGGraph graph = graphItem.Tag as SLGGraph;
            IEnumerator<SLGOutputDevice> outDevEnum = graph.GetOutputDevicesEnumerator();
            outDevEnum.Reset();
            while (outDevEnum.MoveNext())
            {
                SLGOutputDevice device = outDevEnum.Current;
                string statistics;
                if (device.GetStatistics(out statistics) == HR.S_OK)
                {
                    if (string.IsNullOrEmpty(statistics))
                    {
                        MessageBox.Show(Window.GetWindow(this), "There is no statistic", WindowTitle, MessageBoxButton.OK, MessageBoxImage.Error);
                        return;
                    }
                    StatisticsWindow wnd = new StatisticsWindow(device, statistics);
                    wnd.Owner = Window.GetWindow(this);
                    wnd.ShowDialog();
                }
                else
                {
                    MessageBox.Show("Statistic error", WindowTitle, MessageBoxButton.OK, MessageBoxImage.Error);
                }
                return;
            }
            MessageBox.Show("There is no output device", WindowTitle, MessageBoxButton.OK, MessageBoxImage.Error);
            
        }

        private void btnConnect_Click(object sender, RoutedEventArgs e)
        {
            if (_scheduler.IsConnected)
            {
                Disconnect();
            }
            else
            {
                Connect();
            }
        }

        private void this_Closed(object sender, EventArgs e)
        {
            Disconnect();
            try
            {
                using (RegistryKey key = Registry.CurrentUser.CreateSubKey("Software\\SoftLab-NSK\\SLStreamerTest"))
                {
                    key.SetValue("SchedulerAddress", this.SchedulerAddress);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, this.Title, MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

            
    }
}
